#include "xincs.h"
#include "fxver.h"
#include "fxdefs.h"
#include "FXStream.h"
#include "FXString.h"
#include "FXSize.h"
#include "FXId.h"
#include "FXPoint.h"
#include "FXRectangle.h"
#include "FXSettings.h"
#include "FXRegistry.h"
#include "FXAccelTable.h"
#include "FXApp.h"

#include "FXString.h"

#include "FXALDevice.h"

// Object implementation
FXIMPLEMENT(FXALDevice,FXId,NULL,0)

FXALDevice::FXALDevice(FXApp * a,FXchar * deviceid) : FXId(a)
{
	FXTRACE((100,"FXALDevice::FXALDevice %p\n",this));
	
	deviceid = deviceid;
	dev = NULL;
}

FXALDevice::~FXALDevice()
{
	FXTRACE((100,"FXALDevice::~FXALDevice %p\n",this));
	
	destroy();
}


void FXALDevice::attach(FXchar * di)
{
	FXTRACE((100,"FXALDevice::attach %p\n",this));
	
	deviceid = di;
	create();
}

void FXALDevice::create()
{

#ifdef HAVE_OPENAL

	FXTRACE((100,"FXALDevice::create %p\n",this));

	//close possible exsisting device.
	destroy();
	

	//although not preferred, programs may hardcore the device in their program
	//try specified device.
	if (!deviceid.empty())
	{
		dev = alcOpenDevice((FXuchar*)deviceid.text());
		if (dev) return;
	}
	
	//try device entry from Register
	if (getApp())
	{
		deviceid = getApp()->reg().readStringEntry("OpenAL","deviceid",NULL);
		if (!deviceid.empty())
		{
			dev = alcOpenDevice((FXuchar*)deviceid.text());
			if (dev) return;
		}
	}

	//try default device when nothing was specified or requested device failed.
	dev = alcOpenDevice(NULL);
	if (dev) return;

	//Oke... somehow we cannot open any of the devices....
	//should we give some sort of error?

#endif

}


void FXALDevice::destroy()
{
#ifdef HAVE_OPENAL

	FXTRACE((100,"FXALDevice::destroy %p\n",this));

	//close possible exsisting device.
	if (dev) alcCloseDevice((ALCdevice*)dev);
	dev = NULL;

#endif

}
